<?php
/*
Plugin Name: Above & Below Admin Tables
Plugin URI: https://solarisedesign.co.uk/adding-content-above-and-below-the-wordpress-admin-posts-table
Description: Place content above and below the 'All Posts' index tables within Wordpress
Version: 0.0.0
Author: Robin Metcalfe
Author URI: http://solarisedesign.co.uk
*/

defined( 'ABSPATH' ) or die( 'No direct access' );

class AboveBelowAdminTables {

	// This tells us if we're currently on the 'All Posts' screen
	// This is populated during the 'current_screen' action hook
	protected $onAllScreen = false;
	// Also grab the post_type data
	protected $postType = null;

	/**
	 * Register our action hook handlers with WP
	 */
	function __construct()
	{
		add_action('current_screen', array($this, 'getCurrentScreen'));
		add_action('admin_head', array($this, 'injectScript'));
	}

	/**
	 * Populate the $onAllScreen boolean and $postType vars
	 */
	function getCurrentScreen($screen)
	{
		if(!is_admin()) {
			return;
		}

		if($screen->base == 'edit') {
			$this->onAllScreen = true;
			$this->postType = $screen->post_type;
		}
	}

	/**
	 * Prints out the necessary <template> tags, CSS and Javascript into
	 * the admin <head> element
	 */
	function injectScript()
	{

		$templates = array();
		$templateHtml = array();

		// Locate the 'above' and 'below' template files for this post type
		$aboveFile = locate_template("above-below-admin-tables/{$this->postType}-above-all.php");
		$belowFile = locate_template("above-below-admin-tables/{$this->postType}-below-all.php");

		// Grab the contents of these files. Note we're using object buffering to ensure
		// that the PHP within is executed
		if(is_readable($aboveFile)) {
			ob_start();
			require_once($aboveFile);
			$templates['above'] = ob_get_clean();
		}

		if(is_readable($belowFile)) {
			ob_start();
			require_once($belowFile);
			$templates['below'] = ob_get_clean();
		}

		// Run through the template content and prepare the <template>
		// tags containing the relevant HTML content (our Javascript will
		// use these to pull our imported content into the correct
		// container elements)
		foreach($templates as $location => $html) {
			$templateHtml[] = <<<HTML
			<template name='{$location}'>
				{$html}
			</template>
HTML;
		}

		$templateHtmlComplete = implode("\n", $templateHtml);

		echo <<<HTML
		<style>
			.custom-admin-table-content {
				border: 1px solid #e1e1e1;
				color: #32373c;
				background: #FFF;
				border-left: 4px solid #888;
				padding: 0 12px;
			}
			/* for the situation where there are no additional
			   notification boxes above */
			.wp-header-end + .custom-admin-table-content {
				margin-top: 1em;
			}
			.custom-admin-table-content p {
				margin: 0.5em 0;
				padding: 2px;
			}
			.custom-admin-table-content.below {
				margin-top: 1em;
			}
		</style>

		<!-- Admin header/footer table templates for this post type -->
		{$templateHtmlComplete}

		<script>

			// encapsulated so we don't interfere with global scope
			var AboveBelowAdminTables = {
				init: function(){
					// Construct the container to add to the top
					var aboveContainer = document.createElement("div");
					var belowContainer = document.createElement("div");
					
					aboveContainer.classList.add('custom-admin-table-content', 'above');
					belowContainer.classList.add('custom-admin-table-content', 'below');
					
					// These templates are generated from the output of the PHP template files, placed earlier in the <head> output
					var aboveTemplate = document.querySelector('template[name=above]');
					var belowTemplate = document.querySelector('template[name=below]');

					// If the templates exist, apply their contents to the new
					// container elements
					if(aboveTemplate) {
						aboveContainer.innerHTML = aboveTemplate.innerHTML;
					}

					if(belowTemplate) {
						belowContainer.innerHTML = belowTemplate.innerHTML;
					}

					// Insert a 0 timeout to cater for initial JS initialization
					setTimeout(function(){

						if(aboveTemplate) {
							// .subsubsub is the class of the container holding the All(123) | Published(2) | Trash(5) status
							var targetAbove = document.querySelector(".subsubsub");						
							if(targetAbove) {
								// Insert the "above" container just before .subsubsub
								targetAbove.parentNode.insertBefore(aboveContainer, targetAbove.previousSibling);
							}
						}

						if(belowTemplate) {
							// We'll plop the belowContainer right after the bottom navigation element
							// of the table
							var targetBelow = document.querySelector('.tablenav.bottom');
							if(targetBelow) {
								targetBelow.parentNode.appendChild(belowContainer);
							}
						}

					}, 0)
					
				}
			}

			// Let's go
			AboveBelowAdminTables.init();


		</script>
HTML;
	}

}

new AboveBelowAdminTables;